//
// Core B start up code
//

#include <defBF561.h>
#include "config.h"


.global start_b

start_b:
	sp.h = 0xff70;		//Set up supervisor stack in scratch pad
	sp.l = 0x0800;
	fp = sp;

// Initialize Exception handlers
	p0.l = LO(EVT2);
	p0.h = HI(EVT2);

	r0.l = _NHANDLER;
	r0.h = _NHANDLER;  	// NMI Handler (Int2)
    [p0++] = r0;

    r0.l = EXC_HANDLER;
  	r0.h = EXC_HANDLER;  	// Exception Handler (Int3)
    [p0++] = r0;
	
	[p0++] = r0; 		// IVT4 isn't used

    r0.l = _HWHANDLER;
	r0.h = _HWHANDLER; 	// HW Error Handler (Int5)
    [p0++] = r0;
	
    r0.l = _THANDLER;
	r0.h = _THANDLER;  	// Timer Handler (Int6)
	[p0++] = r0;
	
    r0.l = _RTCHANDLER;
	r0.h = _RTCHANDLER; // IVG7 Handler
	[p0++] = r0;
	
    r0.l = _I8HANDLER;
	r0.h = _I8HANDLER; 	// IVG8 Handler
  	[p0++] = r0;
  	
  	r0.l = _I9HANDLER;
	r0.h = _I9HANDLER; 	// IVG9 Handler
 	[p0++] = r0;
 	
    r0.l = _I10HANDLER;
	r0.h = _I10HANDLER;	// IVG10 Handler
 	[p0++] = r0;
 	
    r0.l = _I11HANDLER;
	r0.h = _I11HANDLER;	// IVG11 Handler
  	[p0++] = r0;
  	
    r0.l = _I12HANDLER;
	r0.h = _I12HANDLER;	// IVG12 Handler
  	[p0++] = r0;
  	
    r0.l = _I13HANDLER;
	r0.h = _I13HANDLER;	// IVG13 Handler
    [p0++] = r0;

    r0.l = _I14HANDLER;
	r0.h = _I14HANDLER;	// IVG14 Handler
  	[p0++] = r0;

    r0.l = _I15HANDLER;
	r0.h = _I15HANDLER;	// IVG15 Handler
	[p0++] = r0;




	//  Enable Interrupt 15 
	p0.l = LO(EVT15);
	p0.h = HI(EVT15);
	r0.l = call_main;  // install isr 15 as caller to main
	r0.h = call_main;
	[p0] = r0;

	r0 = 0x8000(z);    // enable irq 15 only
	sti r0;            // set mask
	raise 15;          // raise sw interrupt
	
	p0.l = wait;
	p0.h = wait;

	reti = p0;
	rti;               // return from reset

wait:
	jump wait;         // wait until irq 15 is being serviced.


call_main:
	[--sp] = reti;  // pushing RETI allows interrupts to occur inside all main routines

	p0.l = _coreB_main;
	p0.h = _coreB_main;

	r0.l = end;
	r0.h = end;

	rets = r0;      // return address for main()'s RTS

	jump (p0);

end:
	idle;
	jump end;


start_b.end:


////////////////////////////////////////////////////////////////////////////
// Default handlers:
//


// If we get caught in one of these handlers for some reason, 
// display the IRQ vector on the EZKIT LEDs and enter
// endless loop.

.global b_idle_loop
b_idle_loop:
	idle;
	jump b_idle_loop;


_HWHANDLER:           // HW Error Handler 5
	jump b_idle_loop;

_NHANDLER:
	jump b_idle_loop;

EXC_HANDLER:          // exception handler
	jump b_idle_loop;

_THANDLER:            // Timer Handler 6	
	r0.l = 6;
	jump b_idle_loop;

_RTCHANDLER:          // IVG 7 Handler  
	r0.l = 7;
	jump b_idle_loop;

_I8HANDLER:           // IVG 8 Handler
	r0.l = 8;
	jump b_idle_loop;

_I9HANDLER:           // IVG 9 Handler
	r0.l = 9;
	jump b_idle_loop;

_I10HANDLER:          // IVG 10 Handler
	r0.l = 10;
	jump b_idle_loop;

_I11HANDLER:          // IVG 11 Handler
	r0.l = 11;
	jump b_idle_loop;

_I12HANDLER:          // IVG 12 Handler
	r0.l = 12;
	jump b_idle_loop;

_I13HANDLER:		  // IVG 13 Handler
	r0.l = 13;
	jump b_idle_loop;
 
_I14HANDLER:		  // IVG 14 Handler
	r0.l = 14;
	jump b_idle_loop;

_I15HANDLER:		  // IVG 15 Handler
	r0.l = 15;
	jump b_idle_loop;

